library(tidyverse)
library(lattice)
library(viridisLite)
library(Rfast)
library(gridExtra)
library(reshape2)
library(ggplot2)
library(scales)

#-----------------------------------------------------------
#1. Heat Maps for 3 Guild Variables 
load("guildinfodata_0522.RData")

guildinfodata<-rbind(guildinfodata_0522[1:49,],
                     c(50,"2011-01-29",1,50,rep(0,5)),
                     guildinfodata_0522[50:1549,])

K<-50
guild_size<-matrix(0,K,31)
guild_spend<-matrix(0,K,31)
guild_meminteract<-matrix(0,K,31)
guild_noofpurch<-matrix(0,K,31)
guild_avggame<-matrix(0,K,31)

for(i in 1:K){
  
  temp<-data.frame(guildinfodata[guildinfodata$guildindex==i,
                                 5:9],rownames.force = NA)
  
  temp[is.na(temp)]<-0
  
  guild_size[i,]<-as.numeric(temp[,5])
  guild_spend[i,]<-as.numeric(temp[,3])
  guild_meminteract[i,]<-as.numeric(temp[,1])
  guild_noofpurch[i,]<-as.numeric(temp[,4])
  guild_avggame[i,]<-as.numeric(temp[,2])
}
rm('guildinfodata_0522','temp')

x.scale <- list(at=seq(0,30,5))

coul <- heat.colors(100)
p1<-levelplot(t(guild_meminteract[1:(K-1),1:30]),at=seq(0, 50, length.out=30),
              col.regions=coul,
              scales=list(x=x.scale),xlab="Days",ylab="Guilds",main="No. of guild members who played in team")

p2<-levelplot(t(guild_avggame[1:(K-1),1:30]),at=seq(0, 8, length.out=30),
              col.regions=coul,
              scales=list(x=x.scale),xlab="Days",ylab="Guilds",main="No. of daily average game sessions")

p3<-levelplot(t(guild_noofpurch[1:(K-1),1:30]),at=seq(0, 10, length.out=30),
          col.regions=coul,
          scales=list(x=x.scale),xlab="Days",ylab="Guilds",main="No. of purchases by guild members")

grid.arrange(p1,p2,p3,ncol=3)

#--------------------------------------------------------------

id.remove<-c('412707','626494','632811','981252',
             '987762','8710791','9569599','9600937',
             '10182266','10360113','10502933','10632317',
             '10638416','10786900','10921298','10987066',
             '11509405','18892676','19201223','19222498')

#2.Now with matrix x of player specific variables
load("data_0522.RData")
iv<-data_0522
for(ii in 1:length(id.remove)){
  
  idx<-which(iv$characterid==id.remove[ii])
  iv<-iv[-idx,]
}

#3. Get login ind. and login time
library(readr)
dv_0522 <- read_csv("dv_0522.txt")
dv<-dv_0522
for(ii in 1:length(id.remove)){
  
  idx<-which(dv$characterid==id.remove[ii])
  dv<-dv[-idx,]
}

#4. Do some clean up

#first: logindummy=1 and logintime = 0. Replace logintime with 1
dv$logintime[dv$logindummy==1 & dv$logintime==0]<- 1

#second: logindummy = 0 and pvpplaytime or pvpkillpoint >0. Replace logindummy = 1
# and logintime = max(1,pvpplaytime+pveplaytime)
data.full<-cbind(iv,dv)
data.full[is.na(data.full)]<-0
data.full$logindummy[(data.full$pvpplaytime>0 | data.full$pvpkillpoint>0) & data.full$logindummy==0]<- 1
data.full$logintime[data.full$logindummy==1 & data.full$logintime==0]<- data.full$pvpplaytime[data.full$logindummy==1 & data.full$logintime==0]+
  data.full$pvetime[data.full$logindummy==1 & data.full$logintime==0]
data.full$logintime[data.full$logindummy==1 & data.full$logintime==0]<- 1

#third: level should be 1 for the players above
data.full$level[data.full$logindummy==1 & data.full$level==0]<- 1
data.full$friendmeanlevel[data.full$logindummy==1 & data.full$friendmeanlevel==0]<- 1
rm('data_0522','dv_0522')

#4. Add new variables (time_since, week, holiday)
mydate<- unique(as.Date(data.full$date,format="%Y-%m-%d"))
week<- matrix(0,length(mydate),1)
week<- 1*(weekdays(mydate)=="Saturday" | weekdays(mydate)=="Sunday")
holiday<- matrix(0,length(mydate),1)
holiday[mydate=="2011-02-03" | mydate=="2011-02-14"]<- 1

# Get the Guildmembership matrix
load("data_0522_guildmemship.RData")
guild_mem<- data_0522_guildmemship
rm('data_0522_guildmemship')
for(ii in 1:length(id.remove)){
  
  idx<-which(guild_mem$characterid==id.remove[ii])
  guild_mem<-guild_mem[-idx,]
}
K<-50
gm.all<-guild_mem
rm('guild_mem')
idx<-(gm.all$guilddummy>0)
gm.all<-gm.all[idx,]
rm('idx')

new_dv<-cbind(data.full,gm.all$characterid,gm.all$guildindex)
rm('gm.all')
rm('dv')

m<-31
plotmat_login<-matrix(0,K,m)
plotmat_purch<-matrix(0,K,m)
plotmat_act<-matrix(0,K,m)

for(k in 1:K){
  for(j in 1:m){
  
  temp<-new_dv[new_dv$dayid==j & new_dv$`gm.all$guildindex`==k,]
  plotmat_login[k,j] = 100*sum(temp$logindummy)/(dim(temp)[1])
  plotmat_purch[k,j] = 100*sum(temp$purchasedummy)/sum(temp$logindummy)
  plotmat_act[k,j] = sum(temp$logintime)/(60*sum(temp$logindummy))
  }
}

plotmat_login<- plotmat_login[1:(K-1),1:(m-1)]
rownames(plotmat_login) = paste("guild", seq(K-1), sep="")
colnames(plotmat_login) = paste("day", seq(m-1), sep="")
dat_login = as.data.frame(plotmat_login)
dat_login$guild = rownames(dat_login)
mdat_login = melt(dat_login, id.vars="guild")
mdat_login$day = as.numeric(gsub("day", "", mdat_login$variable))

plotmat_purch[plotmat_purch==0]<- 0.01
plotmat_purch<- plotmat_purch[1:(K-1),1:(m-1)]
rownames(plotmat_purch) = paste("guild", seq(K-1), sep="")
colnames(plotmat_purch) = paste("day", seq(m-1), sep="")
dat_purch = as.data.frame(plotmat_purch)
dat_purch$guild = rownames(dat_purch)
mdat_purch = melt(dat_purch, id.vars="guild")
mdat_purch$day = as.numeric(gsub("day", "", mdat_purch$variable))

plotmat_act<- plotmat_act[1:(K-1),1:(m-1)]
rownames(plotmat_act) = paste("guild", seq(K-1), sep="")
colnames(plotmat_act) = paste("day", seq(m-1), sep="")
dat_act = as.data.frame(plotmat_act)
dat_act$guild = rownames(dat_act)
mdat_act = melt(dat_act, id.vars="guild")
mdat_act$day = as.numeric(gsub("day", "", mdat_act$variable))

g1 = ggplot()+theme_bw()+ylab("% of Players Logged In")+
  xlab("Days")+
  geom_smooth(data=mdat_login, aes(x=day, y=value, group=guild),
              color='violet',size=0.5,span=0.2,se=FALSE)+
  geom_smooth(data=mdat_login, aes(x=day, y=value),
              color='red',size=2, alpha=0.5,span=0.01)+
  scale_x_continuous(breaks = seq(0,30,2))+
  geom_vline(xintercept = which(week==1), 
             color = "black", size=0.1)+
  geom_vline(xintercept = which(holiday==1),linetype="dotted", 
             color = "blue", size=0.1)
  

g2 = ggplot() + theme_bw() + 
  ylab("% of Players with Purchase >0")+xlab("Days")+
  geom_smooth(data=mdat_purch, aes(x=day, y=value, group=guild),
              color='orange',size=0.5,alpha = 0.2,span=0.2,se=FALSE)+
  geom_smooth(data=mdat_purch, aes(x=day, y=value),
              color='red',size=2, alpha=0.5,span=0.01)+
  scale_x_continuous(breaks = seq(0,30,2))+
  scale_y_continuous(limit=c(0,NA),oob=squish)+
  geom_vline(xintercept = which(week==1), 
             color = "black", size=0.1)+
  geom_vline(xintercept = which(holiday==1),linetype="dotted", 
             color = "blue", size=0.1)

g3 = ggplot() + theme_bw() + scale_x_continuous(breaks = seq(0,30,2))+
  ylab("Duration (minutes)")+xlab("Days")+
  geom_smooth(data=mdat_act, aes(x=day, y=value, group=guild),
            color='green',size=0.5,alpha = 0.2,span=0.2,se=FALSE)+
  geom_smooth(data=mdat_act, aes(x=day, y=value),
              color='red',size=2, alpha=0.5,span=0.01)+
  geom_vline(xintercept = which(week==1), 
             color = "black", size=0.1)+
  geom_vline(xintercept = which(holiday==1),linetype="dotted", 
             color = "blue", size=0.1)

grid.arrange(g1,g3,g2,ncol=3)

#---- Dependent variable plots (Figures 7 and 8 in Supplement A) ---------------

#Add new variable - time_since
new_dv$timesince<- 0
unique.id<-unique(new_dv$characterid)
n<- length(unique.id)

for(i in 1:n){
  
  ii<-unique.id[i]
  temp1<- new_dv[new_dv$characterid==ii,c('logindummy','dayid')]
  temp2<- matrix(0,m)
  for(j in 2:(m)){
    
    if(temp1$logindummy[j-1]>0){
      temp2[j]<- 0
    }
    if(temp1$logindummy[j-1]==0){
      temp2[j]<- 1+temp2[j-1]
    }
  }
  new_dv$timesince[new_dv$characterid==ii]<- temp2
}
rm('temp1','temp2')


plotmat_level<-matrix(0,K,m)
plotmat_pvptime<-matrix(0,K,m)
plotmat_pvpkill<-matrix(0,K,m)
plotmat_questcount<-matrix(0,K,m)
plotmat_pvetime<-matrix(0,K,m)
plotmat_noofgames<-matrix(0,K,m)
plotmat_frndcount<-matrix(0,K,m)
plotmat_frndlevel<-matrix(0,K,m)
plotmat_frndinteract<-matrix(0,K,m) 
plotmat_timesince<-matrix(0,K,m)

plotmat_mission<-matrix(0,K,m)
plotmat_nfrndpurch<-matrix(0,K,m)
plotmat_frndpurchval<-matrix(0,K,m)
plotmat_gamefrnd<-matrix(0,K,m)
plotmat_guildtenure<-matrix(0,K,m)
plotmat_nguildintera<-matrix(0,K,m)
plotmat_ngameguild<-matrix(0,K,m)

for(k in 1:K){
  for(j in 1:m){
    
    temp<-new_dv[new_dv$dayid==j & new_dv$`gm.all$guildindex`==k & new_dv$logindummy==1,]
    plotmat_level[k,j] = sum(temp$level)/(dim(temp)[1])
    plotmat_pvptime[k,j] = sum(temp$pvpplaytime/60)/(dim(temp)[1])
    plotmat_pvpkill[k,j] = sum(temp$pvpkillpoint)/(dim(temp)[1])
    plotmat_questcount[k,j] = sum(temp$questid)/(dim(temp)[1])
    plotmat_pvetime[k,j] = sum(temp$pvetime/60)/(dim(temp)[1])
    plotmat_noofgames[k,j] = sum(temp$numberofgame)/(dim(temp)[1])
    plotmat_frndcount[k,j] = sum(temp$friendcount.1)/(dim(temp)[1])
    plotmat_frndlevel[k,j] = sum(temp$friendmeanlevel)/(dim(temp)[1])
    plotmat_frndinteract[k,j] = sum(temp$numberoffriendinteract)/(dim(temp)[1])
    temp<-new_dv[new_dv$dayid==j & new_dv$`gm.all$guildindex`==k,]
    plotmat_timesince[k,j] = sum(temp$timesince)/(dim(temp)[1])
    
    plotmat_mission[k,j]<-sum(temp$missionid)/(dim(temp)[1])
    plotmat_nfrndpurch[k,j]<-sum(temp$nofriendpurchase)/(dim(temp)[1])
    plotmat_frndpurchval[k,j]<-sum(temp$totalfriendbuy)/(dim(temp)[1])
    plotmat_gamefrnd[k,j]<-sum(temp$gameroundplaywithfriends)/(dim(temp)[1])
    plotmat_guildtenure[k,j]<-sum(temp$guildtenure)/(dim(temp)[1])
    plotmat_nguildintera[k,j]<-sum(temp$noofguildmeminteract)/(dim(temp)[1])
    plotmat_ngameguild[k,j]<-sum(temp$noofgamewithguildmem)/(dim(temp)[1])
  }
}

plotmat_level<- plotmat_level[1:(K-1),1:(m-1)]
rownames(plotmat_level) = paste("guild", seq(K-1), sep="")
colnames(plotmat_level) = paste("day", seq(m-1), sep="")
dat_level = as.data.frame(plotmat_level)
dat_level$guild = rownames(dat_level)
mdat_level = melt(dat_level, id.vars="guild")
mdat_level$day = as.numeric(gsub("day", "", mdat_level$variable))

plotmat_pvptime<- plotmat_pvptime[1:(K-1),1:(m-1)]
rownames(plotmat_pvptime) = paste("guild", seq(K-1), sep="")
colnames(plotmat_pvptime) = paste("day", seq(m-1), sep="")
dat_pvptime = as.data.frame(plotmat_pvptime)
dat_pvptime$guild = rownames(dat_pvptime)
mdat_pvptime = melt(dat_pvptime, id.vars="guild")
mdat_pvptime$day = as.numeric(gsub("day", "", mdat_pvptime$variable))

plotmat_pvpkill<- plotmat_pvpkill[1:(K-1),1:(m-1)]
rownames(plotmat_pvpkill) = paste("guild", seq(K-1), sep="")
colnames(plotmat_pvpkill) = paste("day", seq(m-1), sep="")
dat_pvpkill = as.data.frame(plotmat_pvpkill)
dat_pvpkill$guild = rownames(dat_pvpkill)
mdat_pvpkill = melt(dat_pvpkill, id.vars="guild")
mdat_pvpkill$day = as.numeric(gsub("day", "", mdat_pvpkill$variable))

plotmat_questcount<- plotmat_questcount[1:(K-1),1:(m-1)]
rownames(plotmat_questcount) = paste("guild", seq(K-1), sep="")
colnames(plotmat_questcount) = paste("day", seq(m-1), sep="")
dat_questcount = as.data.frame(plotmat_questcount)
dat_questcount$guild = rownames(dat_questcount)
mdat_questcount = melt(dat_questcount, id.vars="guild")
mdat_questcount$day = as.numeric(gsub("day", "", mdat_questcount$variable))

plotmat_pvetime<- plotmat_pvetime[1:(K-1),1:(m-1)]
rownames(plotmat_pvetime) = paste("guild", seq(K-1), sep="")
colnames(plotmat_pvetime) = paste("day", seq(m-1), sep="")
dat_pvetime = as.data.frame(plotmat_pvetime)
dat_pvetime$guild = rownames(dat_pvetime)
mdat_pvetime = melt(dat_pvetime, id.vars="guild")
mdat_pvetime$day = as.numeric(gsub("day", "", mdat_pvetime$variable))

plotmat_noofgames<- plotmat_noofgames[1:(K-1),1:(m-1)]
rownames(plotmat_noofgames) = paste("guild", seq(K-1), sep="")
colnames(plotmat_noofgames) = paste("day", seq(m-1), sep="")
dat_noofgames = as.data.frame(plotmat_noofgames)
dat_noofgames$guild = rownames(dat_noofgames)
mdat_noofgames = melt(dat_noofgames, id.vars="guild")
mdat_noofgames$day = as.numeric(gsub("day", "", mdat_noofgames$variable))

plotmat_frndcount<- plotmat_frndcount[1:(K-1),1:(m-1)]
rownames(plotmat_frndcount) = paste("guild", seq(K-1), sep="")
colnames(plotmat_frndcount) = paste("day", seq(m-1), sep="")
dat_frndcount = as.data.frame(plotmat_frndcount)
dat_frndcount$guild = rownames(dat_frndcount)
mdat_frndcount = melt(dat_frndcount, id.vars="guild")
mdat_frndcount$day = as.numeric(gsub("day", "", mdat_frndcount$variable))

plotmat_frndlevel<- plotmat_frndlevel[1:(K-1),1:(m-1)]
rownames(plotmat_frndlevel) = paste("guild", seq(K-1), sep="")
colnames(plotmat_frndlevel) = paste("day", seq(m-1), sep="")
dat_frndlevel = as.data.frame(plotmat_frndlevel)
dat_frndlevel$guild = rownames(dat_frndlevel)
mdat_frndlevel = melt(dat_frndlevel, id.vars="guild")
mdat_frndlevel$day = as.numeric(gsub("day", "", mdat_frndlevel$variable))

plotmat_frndinteract<- plotmat_frndinteract[1:(K-1),1:(m-1)]
rownames(plotmat_frndinteract) = paste("guild", seq(K-1), sep="")
colnames(plotmat_frndinteract) = paste("day", seq(m-1), sep="")
dat_frndinteract = as.data.frame(plotmat_frndinteract)
dat_frndinteract$guild = rownames(dat_frndinteract)
mdat_frndinteract = melt(dat_frndinteract, id.vars="guild")
mdat_frndinteract$day = as.numeric(gsub("day", "", mdat_frndinteract$variable))

plotmat_timesince<- plotmat_timesince[1:(K-1),1:(m-1)]
rownames(plotmat_timesince) = paste("guild", seq(K-1), sep="")
colnames(plotmat_timesince) = paste("day", seq(m-1), sep="")
dat_timesince = as.data.frame(plotmat_timesince)
dat_timesince$guild = rownames(dat_timesince)
mdat_timesince = melt(dat_timesince, id.vars="guild")
mdat_timesince$day = as.numeric(gsub("day", "", mdat_timesince$variable))

plotmat_mission<-plotmat_mission[1:(K-1),1:(m-1)]
rownames(plotmat_mission) = paste("guild", seq(K-1), sep="")
colnames(plotmat_mission) = paste("day", seq(m-1), sep="")
dat_mission = as.data.frame(plotmat_mission)
dat_mission$guild = rownames(dat_mission)
mdat_mission = melt(dat_mission, id.vars="guild")
mdat_mission$day = as.numeric(gsub("day", "", mdat_mission$variable))

plotmat_nfrndpurch<-plotmat_nfrndpurch[1:(K-1),1:(m-1)]
rownames(plotmat_nfrndpurch) = paste("guild", seq(K-1), sep="")
colnames(plotmat_nfrndpurch) = paste("day", seq(m-1), sep="")
dat_nfrndpurch = as.data.frame(plotmat_nfrndpurch)
dat_nfrndpurch$guild = rownames(dat_nfrndpurch)
mdat_nfrndpurch = melt(dat_nfrndpurch, id.vars="guild")
mdat_nfrndpurch$day = as.numeric(gsub("day", "", mdat_nfrndpurch$variable))

plotmat_frndpurchval<-plotmat_frndpurchval[1:(K-1),1:(m-1)]
rownames(plotmat_frndpurchval) = paste("guild", seq(K-1), sep="")
colnames(plotmat_frndpurchval) = paste("day", seq(m-1), sep="")
dat_frndpurchval = as.data.frame(plotmat_frndpurchval)
dat_frndpurchval$guild = rownames(dat_frndpurchval)
mdat_frndpurchval = melt(dat_frndpurchval, id.vars="guild")
mdat_frndpurchval$day = as.numeric(gsub("day", "", mdat_frndpurchval$variable))

plotmat_gamefrnd<-plotmat_gamefrnd[1:(K-1),1:(m-1)]
rownames(plotmat_gamefrnd) = paste("guild", seq(K-1), sep="")
colnames(plotmat_gamefrnd) = paste("day", seq(m-1), sep="")
dat_gamefrnd = as.data.frame(plotmat_gamefrnd)
dat_gamefrnd$guild = rownames(dat_gamefrnd)
mdat_gamefrnd = melt(dat_gamefrnd, id.vars="guild")
mdat_gamefrnd$day = as.numeric(gsub("day", "", mdat_gamefrnd$variable))

plotmat_guildtenure<-plotmat_guildtenure[1:(K-1),1:(m-1)]
rownames(plotmat_guildtenure) = paste("guild", seq(K-1), sep="")
colnames(plotmat_guildtenure) = paste("day", seq(m-1), sep="")
dat_guildtenure = as.data.frame(plotmat_guildtenure)
dat_guildtenure$guild = rownames(dat_guildtenure)
mdat_guildtenure = melt(dat_guildtenure, id.vars="guild")
mdat_guildtenure$day = as.numeric(gsub("day", "", mdat_guildtenure$variable))

plotmat_nguildintera<-plotmat_nguildintera[1:(K-1),1:(m-1)]
rownames(plotmat_nguildintera) = paste("guild", seq(K-1), sep="")
colnames(plotmat_nguildintera) = paste("day", seq(m-1), sep="")
dat_nguildintera = as.data.frame(plotmat_nguildintera)
dat_nguildintera$guild = rownames(dat_nguildintera)
mdat_nguildintera = melt(dat_nguildintera, id.vars="guild")
mdat_nguildintera$day = as.numeric(gsub("day", "", mdat_nguildintera$variable))

plotmat_ngameguild<-plotmat_ngameguild[1:(K-1),1:(m-1)]
rownames(plotmat_ngameguild) = paste("guild", seq(K-1), sep="")
colnames(plotmat_ngameguild) = paste("day", seq(m-1), sep="")
dat_ngameguild = as.data.frame(plotmat_ngameguild)
dat_ngameguild$guild = rownames(dat_ngameguild)
mdat_ngameguild = melt(dat_ngameguild, id.vars="guild")
mdat_ngameguild$day = as.numeric(gsub("day", "", mdat_ngameguild$variable))


g4 = ggplot()+theme_bw()+ylab("Average Level")+
  xlab("Days")+
  geom_smooth(data=mdat_level, aes(x=day, y=value, group=guild),
              color='gray',size=0.5,span=0.2,se=FALSE)+
  geom_smooth(data=mdat_level, aes(x=day, y=value),
              color='black',size=2, alpha=0.5,span=0.01,se=FALSE)+
  scale_x_continuous(breaks = seq(0,30,2))+
  geom_vline(xintercept = which(week==1), 
             color = "black", size=0.1)+
  geom_vline(xintercept = which(holiday==1),linetype="dashed", 
             color = "black", size=0.2)+
  theme(axis.text.x = element_text(size=15),
        axis.text.y = element_text(size=15),
        axis.title.x = element_text(size=15),
        axis.title.y = element_text(size=15))

g5 = ggplot()+theme_bw()+ylab("Average PVP Playtime (minutes)")+
  xlab("Days")+
  geom_smooth(data=mdat_pvptime, aes(x=day, y=value, group=guild),
              color='gray',size=0.5,span=0.1,se=FALSE)+
  geom_smooth(data=mdat_pvptime, aes(x=day, y=value),
              color='black',size=2, alpha=0.5,span=0.01,se=FALSE)+
  scale_x_continuous(breaks = seq(0,30,2))+
  geom_vline(xintercept = which(week==1), 
             color = "black", size=0.1)+
  geom_vline(xintercept = which(holiday==1),linetype="dashed", 
             color = "black", size=0.2)+
  theme(axis.text.x = element_text(size=15),
        axis.text.y = element_text(size=15),
        axis.title.x = element_text(size=15),
        axis.title.y = element_text(size=15))

g6 = ggplot()+theme_bw()+ylab("Average PVP Kill Points")+
  xlab("Days")+
  geom_smooth(data=mdat_pvpkill, aes(x=day, y=value, group=guild),
              color='gray',size=0.5,span=0.1,se=FALSE)+
  geom_smooth(data=mdat_pvpkill, aes(x=day, y=value),
              color='black',size=2, alpha=0.5,span=0.01,se=FALSE)+
  scale_x_continuous(breaks = seq(0,30,2))+
  geom_vline(xintercept = which(week==1), 
             color = "black", size=0.1)+
  geom_vline(xintercept = which(holiday==1),linetype="dashed", 
             color = "black", size=0.2)+
  theme(axis.text.x = element_text(size=15),
        axis.text.y = element_text(size=15),
        axis.title.x = element_text(size=15),
        axis.title.y = element_text(size=15))

grid.arrange(g4,g5,g6,ncol=3)

g7 = ggplot()+theme_bw()+ylab("Average no. of Quests in PVE mode")+
  xlab("Days")+
  geom_smooth(data=mdat_questcount, aes(x=day, y=value, group=guild),
              color='gray',size=0.5,span=0.2,se=FALSE)+
  geom_smooth(data=mdat_questcount, aes(x=day, y=value),
              color='black',size=2, alpha=0.5,span=0.01,se=FALSE)+
  scale_x_continuous(breaks = seq(0,30,2))+
  geom_vline(xintercept = which(week==1), 
             color = "black", size=0.1)+
  geom_vline(xintercept = which(holiday==1),linetype="dashed", 
             color = "black", size=0.2)+
  theme(axis.text.x = element_text(size=15),
        axis.text.y = element_text(size=15),
        axis.title.x = element_text(size=15),
        axis.title.y = element_text(size=15))

g8 = ggplot()+theme_bw()+ylab("Average PVE Playtime (minutes)")+
  xlab("Days")+
  geom_smooth(data=mdat_pvetime, aes(x=day, y=value, group=guild),
              color='gray',size=0.5,span=0.2,se=FALSE)+
  geom_smooth(data=mdat_pvetime, aes(x=day, y=value),
              color='black',size=2, alpha=0.5,span=0.01,se=FALSE)+
  scale_x_continuous(breaks = seq(0,30,2))+
  geom_vline(xintercept = which(week==1), 
             color = "black", size=0.1)+
  geom_vline(xintercept = which(holiday==1),linetype="dashed", 
             color = "black", size=0.2)+
  theme(axis.text.x = element_text(size=15),
        axis.text.y = element_text(size=15),
        axis.title.x = element_text(size=15),
        axis.title.y = element_text(size=15))

g9 = ggplot()+theme_bw()+ylab("Average PVE games")+
  xlab("Days")+
  geom_smooth(data=mdat_noofgames, aes(x=day, y=value, group=guild),
              color='gray',size=0.5,span=0.2,se=FALSE)+
  geom_smooth(data=mdat_noofgames, aes(x=day, y=value),
              color='black',size=2, alpha=0.5,span=0.01,se=FALSE)+
  scale_x_continuous(breaks = seq(0,30,2))+
  geom_vline(xintercept = which(week==1), 
             color = "black", size=0.1)+
  geom_vline(xintercept = which(holiday==1),linetype="dashed", 
             color = "black", size=0.2)+
  theme(axis.text.x = element_text(size=15),
        axis.text.y = element_text(size=15),
        axis.title.x = element_text(size=15),
        axis.title.y = element_text(size=15))

grid.arrange(g7,g8,g9,ncol=3)

g10 = ggplot()+theme_bw()+ylab("Average Friend Count")+
  xlab("Days")+
  geom_smooth(data=mdat_frndcount, aes(x=day, y=value, group=guild),
              color='gray',size=0.5,span=0.2,se=FALSE)+
  geom_smooth(data=mdat_frndcount, aes(x=day, y=value),
              color='black',size=2, alpha=0.5,span=0.01,se=FALSE)+
  scale_x_continuous(breaks = seq(0,30,2))+
  geom_vline(xintercept = which(week==1), 
             color = "black", size=0.1)+
  geom_vline(xintercept = which(holiday==1),linetype="dashed", 
             color = "black", size=0.2)+
  theme(axis.text.x = element_text(size=15),
        axis.text.y = element_text(size=15),
        axis.title.x = element_text(size=15),
        axis.title.y = element_text(size=15))

g11 = ggplot()+theme_bw()+ylab("Average Friend Level")+
  xlab("Days")+
  geom_smooth(data=mdat_frndlevel, aes(x=day, y=value, group=guild),
              color='gray',size=0.5,span=0.2,se=FALSE)+
  geom_smooth(data=mdat_frndlevel, aes(x=day, y=value),
              color='black',size=2, alpha=0.5,span=0.01,se=FALSE)+
  scale_x_continuous(breaks = seq(0,30,2))+
  geom_vline(xintercept = which(week==1), 
             color = "black", size=0.1)+
  geom_vline(xintercept = which(holiday==1),linetype="dashed", 
             color = "black", size=0.2)+
  theme(axis.text.x = element_text(size=15),
        axis.text.y = element_text(size=15),
        axis.title.x = element_text(size=15),
        axis.title.y = element_text(size=15))

g12 = ggplot()+theme_bw()+ylab("Average no. of Friends Played With")+
  xlab("Days")+
  geom_smooth(data=mdat_frndinteract, aes(x=day, y=value, group=guild),
              color='gray',size=0.5,span=0.2,se=FALSE)+
  geom_smooth(data=mdat_frndinteract, aes(x=day, y=value),
              color='black',size=2, alpha=0.5,span=0.01,se=FALSE)+
  scale_x_continuous(breaks = seq(0,30,2))+
  geom_vline(xintercept = which(week==1), 
             color = "black", size=0.1)+
  geom_vline(xintercept = which(holiday==1),linetype="dashed", 
             color = "black", size=0.2)+
  theme(axis.text.x = element_text(size=15),
        axis.text.y = element_text(size=15),
        axis.title.x = element_text(size=15),
        axis.title.y = element_text(size=15))

grid.arrange(g10,g11,g12,ncol=3)

g13 = ggplot()+theme_bw()+ylab("Average Days Since Last Login")+
  xlab("Days")+
  geom_smooth(data=mdat_timesince, aes(x=day, y=value, group=guild),
              color='gray',size=0.5,span=0.1,se=FALSE)+
  geom_smooth(data=mdat_timesince, aes(x=day, y=value),
              color='black',size=2, alpha=0.5,span=0.01,se=FALSE)+
  scale_x_continuous(breaks = seq(0,30,2))+
  geom_vline(xintercept = which(week==1), 
             color = "black", size=0.1)+
  geom_vline(xintercept = which(holiday==1),linetype="dashed", 
             color = "black", size=0.2)+
  theme(axis.text.x = element_text(size=15),
        axis.text.y = element_text(size=15),
        axis.title.x = element_text(size=15),
        axis.title.y = element_text(size=15))

g14 = ggplot()+theme_bw()+ylab("No. Of Players in guild")+
  xlab("Days")+
  geom_smooth(data=mdat_guild_size, aes(x=day, y=value, group=guild),
              color='gray',size=0.5,span=0.2,se=FALSE)+
  geom_smooth(data=mdat_guild_size, aes(x=day, y=value),
              color='black',size=2, alpha=0.5,span=0.01,se=FALSE)+
  scale_x_continuous(breaks = seq(0,30,2))+
  geom_vline(xintercept = which(week==1), 
             color = "black", size=0.1)+
  geom_vline(xintercept = which(holiday==1),linetype="dashed", 
             color = "black", size=0.2)+
  theme(axis.text.x = element_text(size=15),
        axis.text.y = element_text(size=15),
        axis.title.x = element_text(size=15),
        axis.title.y = element_text(size=15))

g15 = ggplot()+theme_bw()+ylab("Monetary Value of All Purchases in guild")+
  xlab("Days")+
  geom_smooth(data=mdat_guild_spend, aes(x=day, y=value, group=guild),
              color='gray',size=0.5,span=0.1,se=FALSE)+
  geom_smooth(data=mdat_guild_spend, aes(x=day, y=value),
              color='black',size=2, alpha=0.5,span=0.01,se=FALSE)+
  scale_x_continuous(breaks = seq(0,30,2))+
  geom_vline(xintercept = which(week==1), 
             color = "black", size=0.1)+
  geom_vline(xintercept = which(holiday==1),linetype="dashed", 
             color = "black", size=0.2)+
  theme(axis.text.x = element_text(size=15),
        axis.text.y = element_text(size=15),
        axis.title.x = element_text(size=15),
        axis.title.y = element_text(size=15))

grid.arrange(g13,g14,g15,ncol=3)

g16 = ggplot()+theme_bw()+ylab("Average no. of Missions in PVE mode")+
  xlab("Days")+
  geom_smooth(data=mdat_mission, aes(x=day, y=value, group=guild),
              color='gray',size=0.5,span=0.1,se=FALSE)+
  geom_smooth(data=mdat_mission, aes(x=day, y=value),
              color='black',size=2, alpha=0.5,span=0.01,se=FALSE)+
  scale_x_continuous(breaks = seq(0,30,2))+
  geom_vline(xintercept = which(week==1), 
             color = "black", size=0.1)+
  geom_vline(xintercept = which(holiday==1),linetype="dashed", 
             color = "black", size=0.2)+
  theme(axis.text.x = element_text(size=15),
        axis.text.y = element_text(size=15),
        axis.title.x = element_text(size=15),
        axis.title.y = element_text(size=15))

g17 = ggplot()+theme_bw()+ylab("Average no.of times focal player's friends made purchase")+
  xlab("Days")+
  geom_smooth(data=mdat_nfrndpurch, aes(x=day, y=value, group=guild),
              color='gray',size=0.5,span=0.1,se=FALSE)+
  geom_smooth(data=mdat_nfrndpurch, aes(x=day, y=value),
              color='black',size=2, alpha=0.5,span=0.01,se=FALSE)+
  scale_x_continuous(breaks = seq(0,30,2))+
  geom_vline(xintercept = which(week==1), 
             color = "black", size=0.1)+
  geom_vline(xintercept = which(holiday==1),linetype="dashed", 
             color = "black", size=0.2)+
  theme(axis.text.x = element_text(size=15),
        axis.text.y = element_text(size=15),
        axis.title.x = element_text(size=15),
        axis.title.y = element_text(size=15))

g18 = ggplot()+theme_bw()+ylab("Average monetray value of purchases of focal player's friends")+
  xlab("Days")+
  geom_smooth(data=mdat_frndpurchval, aes(x=day, y=value, group=guild),
              color='gray',size=0.5,span=0.1,se=FALSE)+
  geom_smooth(data=mdat_frndpurchval, aes(x=day, y=value),
              color='black',size=2, alpha=0.5,span=0.01,se=FALSE)+
  scale_x_continuous(breaks = seq(0,30,2))+
  geom_vline(xintercept = which(week==1), 
             color = "black", size=0.1)+
  geom_vline(xintercept = which(holiday==1),linetype="dashed", 
             color = "black", size=0.2)+
  theme(axis.text.x = element_text(size=15),
        axis.text.y = element_text(size=15),
        axis.title.x = element_text(size=15),
        axis.title.y = element_text(size=15))

grid.arrange(g16,g17,g18,ncol=3)

g19 = ggplot()+theme_bw()+ylab("Average no. of game sessions played with friends")+
  xlab("Days")+
  geom_smooth(data=mdat_gamefrnd, aes(x=day, y=value, group=guild),
              color='gray',size=0.5,span=0.1,se=FALSE)+
  geom_smooth(data=mdat_gamefrnd, aes(x=day, y=value),
              color='black',size=2, alpha=0.5,span=0.01,se=FALSE)+
  scale_x_continuous(breaks = seq(0,30,2))+
  geom_vline(xintercept = which(week==1), 
             color = "black", size=0.1)+
  geom_vline(xintercept = which(holiday==1),linetype="dashed", 
             color = "black", size=0.2)+
  theme(axis.text.x = element_text(size=15),
        axis.text.y = element_text(size=15),
        axis.title.x = element_text(size=15),
        axis.title.y = element_text(size=15))

g20 = ggplot()+theme_bw()+ylab("Average no. of days a player is member of a guild")+
  xlab("Days")+
  geom_smooth(data=mdat_guildtenure, aes(x=day, y=value, group=guild),
              color='gray',size=0.5,span=0.1,se=FALSE)+
  geom_smooth(data=mdat_guildtenure, aes(x=day, y=value),
              color='black',size=2, alpha=0.5,span=0.01,se=FALSE)+
  scale_x_continuous(breaks = seq(0,30,2))+
  geom_vline(xintercept = which(week==1), 
             color = "black", size=0.1)+
  geom_vline(xintercept = which(holiday==1),linetype="dashed", 
             color = "black", size=0.2)+
  theme(axis.text.x = element_text(size=15),
        axis.text.y = element_text(size=15),
        axis.title.x = element_text(size=15),
        axis.title.y = element_text(size=15))

g21 = ggplot()+theme_bw()+ylab("Average no. of guild members a player played with in teams")+
  xlab("Days")+
  geom_smooth(data=mdat_nguildintera, aes(x=day, y=value, group=guild),
              color='gray',size=0.5,span=0.1,se=FALSE)+
  geom_smooth(data=mdat_nguildintera, aes(x=day, y=value),
              color='black',size=2, alpha=0.5,span=0.01,se=FALSE)+
  scale_x_continuous(breaks = seq(0,30,2))+
  geom_vline(xintercept = which(week==1), 
             color = "black", size=0.1)+
  geom_vline(xintercept = which(holiday==1),linetype="dashed", 
             color = "black", size=0.2)+
  theme(axis.text.x = element_text(size=15),
        axis.text.y = element_text(size=15),
        axis.title.x = element_text(size=15),
        axis.title.y = element_text(size=15))

grid.arrange(g19,g20,g21,ncol=3)

g21 = ggplot()+theme_bw()+ylab("Average no. of game sessions a player played with guild members")+
  xlab("Days")+
  geom_smooth(data=mdat_ngameguild, aes(x=day, y=value, group=guild),
              color='gray',size=0.5,span=0.1,se=FALSE)+
  geom_smooth(data=mdat_ngameguild, aes(x=day, y=value),
              color='black',size=2, alpha=0.5,span=0.01,se=FALSE)+
  scale_x_continuous(breaks = seq(0,30,2))+
  geom_vline(xintercept = which(week==1), 
             color = "black", size=0.1)+
  geom_vline(xintercept = which(holiday==1),linetype="dashed", 
             color = "black", size=0.2)+
  theme(axis.text.x = element_text(size=15),
        axis.text.y = element_text(size=15),
        axis.title.x = element_text(size=15),
        axis.title.y = element_text(size=15))


#------------------------- P matrix ---------------------------------

Pmat<-function(TT,t.val){
  s<-matrix(0,TT,TT)
  for(i in 1:TT){
    s[i,]<-1*(abs(i-c(1:TT))<=t.val)
  }
  e1<-cbind(s,s,s)
  e<-rbind(e1,e1,e1)
  P <- matrix(1, 3*TT, 3*TT)-e
  return(P)
}
TT<- 5
P1<- Pmat(TT,0)
P2<- Pmat(TT,1)
P3<- Pmat(TT,2)

x.scale <- list(at=seq(1,3*TT,1))
out1<-levelplot(P1,col.regions = c(gray(0.75),gray(0.25)),
                scales=list(x=x.scale,y=x.scale),
                xlab="t' = 0",ylab="",colorkey=FALSE)
out2<-levelplot(P2,col.regions = c(gray(0.75),gray(0.25)),
                scales=list(x=x.scale,y=x.scale),
                xlab="t' = 1",ylab="",colorkey=FALSE)
out3<-levelplot(P3,col.regions = c(gray(0.75),gray(0.25)),
                scales=list(x=x.scale,y=x.scale),
                xlab="t' = 2",ylab="",colorkey=FALSE)
grid.arrange(out1,out2,out3,ncol=3)
